@@ -3,6 +3,7 @@ import json
 import logging
 import ssl
 from pathlib import Path
+import xml.etree.ElementTree as ET
 
 import aiofiles
 from bs4 import BeautifulSoup
@@ -58,6 +59,14 @@ class Session:
                     content = json.loads(await response.content.read())
                     return content
 
+    async def get_xml(self, url: URL):
+        async with self.client.simultaneous_session_limit:
+            async with self.rate_limiter:
+                async with self.client_session.get(url, ssl=self.client.ssl_context) as response:
+                    text = await response.content.read()
+                    xmlTree = ET.fromstring(text)
+                    return xmlTree
+
     async def post_no_resp(self, url: URL, headers: dict):
         async with self.client.simultaneous_session_limit:
             async with self.rate_limiter:
@@ -81,7 +90,7 @@ class Session:
         try:
             await self.client_session.close()
         except Exception as e:
-            logging.debug(f"Failed to close sqlite database connection: {str(e)}")
+            logging.debug(f"Failed to close session.")
 
 
 class DownloadSession:
@@ -119,18 +128,18 @@ class DownloadSession:
 
     async def download_file(self, url: URL, referer: str, current_throttle: int, range: str, original_filename: str,
                             filename: str, temp_file: str, resume_point: int, show_progress: bool,
-                            File_Lock: FileLock, folder: Path, title: str):
+                            File_Lock: FileLock, folder: Path, title: str, proxy: str):
         headers = {'Referer': referer, 'user-agent': self.client.user_agent}
         if range:
             headers['Range'] = range
         await throttle(self, current_throttle, url.host)
         async with self.client_session.get(url, headers=headers, ssl=self.client.ssl_context,
-                                           raise_for_status=True) as resp:
+                                           raise_for_status=True, proxy=proxy) as resp:
             content_type = resp.headers.get('Content-Type')
             if 'text' in content_type.lower() or 'html' in content_type.lower():
                 logger.debug("Server for %s is either down or the file no longer exists", str(url))
                 await File_Lock.remove_lock(original_filename)
-                return
+                raise Exception("URL down or not exist")
 
             total = int(resp.headers.get('Content-Length', str(0))) + resume_point
             (folder / title).mkdir(parents=True, exist_ok=True)
