@@ -14,10 +14,11 @@ class BunkrCrawler:
     async def fetch(self, session: Session, url: URL):
         domain_obj = DomainItem(url.host, {})
 
+        ext = '.' + str(url).split('.')[-1]
+        if ext in FILE_FORMATS['Videos']:
+            url = URL(str(url).replace('https://cdn.bunkr.is/', 'https://stream.bunkr.is/v/'))
+
         if await check_direct(url):
-            ext = '.' + str(url).split('.')[-1]
-            if ext in FILE_FORMATS['Videos']:
-                url = URL(str(url).replace('https://cdn', 'https://media-files'))
             await domain_obj.add_to_album(link=url, referral=url, title="Bunkr Loose Files")
             return domain_obj
 
@@ -32,10 +33,13 @@ class BunkrCrawler:
         try:
             soup = await session.get_BS4(url)
             build_id = json.loads(soup.select_one("script[id=__NEXT_DATA__]").get_text())
-
-            json_fetch = URL("https://" + url.host + "/_next/data/" + build_id['buildId'] + url.path + '.json')
-            text = await session.get_text(json_fetch)
-            json_obj = json.loads(text)['pageProps']
+            try:
+                files = build_id['props']['pageProps']['album']['files']
+                json_obj = build_id['props']['pageProps']
+            except KeyError:
+                json_fetch = URL("https://" + url.host + "/_next/data/" + build_id['buildId'] + url.path + '.json')
+                text = await session.get_text(json_fetch)
+                json_obj = json.loads(text)['pageProps']
             title = await make_title_safe(json_obj['album']['name'])
             for file in json_obj['album']['files']:
                 ext = '.' + file['name'].split('.')[-1].lower()
