@@ -1,8 +1,10 @@
 import asyncio
 
 import aiofiles
+from myjdapi import myjdapi
 from yarl import URL
 
+
 from ..client.client import Client
 from ..client.client import Session
 from ..crawlers.Anonfiles_Spider import AnonfilesCrawler
@@ -11,31 +13,37 @@ from ..crawlers.Coomer_Spider import CoomerCrawler
 from ..crawlers.Cyberdrop_Spider import CyberdropCrawler
 from ..crawlers.Cyberfile_Spider import CyberfileCrawler
 from ..crawlers.Erome_Spider import EromeCrawler
-from ..crawlers.XBunker_Spider import XBunkerCrawler
 from ..crawlers.Gfycat_Spider import GfycatCrawler
 from ..crawlers.GoFile_Spider import GofileCrawler
 from ..crawlers.Kemono_Spider import KemonoCrawler
 from ..crawlers.Pixeldrain_Spider import PixelDrainCrawler
 from ..crawlers.Postimg_Spider import PostImgCrawler
 from ..crawlers.Redgifs_Spider import RedGifsCrawler
+from ..crawlers.Rule34_Spider import Rule34Crawler
 from ..crawlers.Saint_Spider import SaintCrawler
 from ..crawlers.ShareX_Spider import ShareXCrawler
 from ..crawlers.SocialMediaGirls_Spider import SocialMediaGirlsCrawler
 from ..crawlers.SimpCity_Spider import SimpCityCrawler
+from ..crawlers.XBunker_Spider import XBunkerCrawler
+from ..crawlers.XBunkr_Spider import XBunkrCrawler
 from ..base_functions.base_functions import log
 from ..base_functions.data_classes import CascadeItem, SkipData
 from ..client.rate_limiting import AsyncRateLimiter
 
 
 class ScrapeMapper():
-    def __init__(self, *, include_id=False, xbunker_auth=None, socialmediagirls_auth=None, simpcity_auth=None,
-                 separate_posts=False, skip_data: SkipData, client: Client, output_last: list):
+    def __init__(self, *, include_id=False, jdownloader_enable=False, jdownloader_device=None, xbunker_auth=None,
+                 socialmediagirls_auth=None, simpcity_auth=None, jdownloader_auth=None, separate_posts=False,
+                 skip_data: SkipData, client: Client, output_last: list):
         self.include_id = include_id
+        self.jdownloader_enable = jdownloader_enable
+        self.jdownloader_device = jdownloader_device
         self.separate_posts = separate_posts
         self.output_last = output_last
         self.xbunker_auth = xbunker_auth
         self.socialmediagirls_auth = socialmediagirls_auth
         self.simpcity_auth = simpcity_auth
+        self.jdownloader_auth = jdownloader_auth
 
         self.client = client
         self.Cascade = CascadeItem({})
@@ -53,23 +61,28 @@ class ScrapeMapper():
         self.pixeldrain_crawler = None
         self.postimg_crawler = None
         self.redgifs_crawler = None
+        self.rule34_crawler = None
         self.saint_crawler = None
         self.sharex_crawler = None
         self.socialmediagirls_crawler = None
         self.simpcity_crawler = None
         self.xbunker_crawler = None
+        self.xbunkr_crawler = None
+
+        self.jdownloader_agent = None
 
         self.jpgchurch_limiter = AsyncRateLimiter(19)
         self.bunkr_limiter = AsyncRateLimiter(15)
         self.forum_limiter = asyncio.Semaphore(4)
         self.semaphore = asyncio.Semaphore(1)
-        self.mapping = {"anonfiles.com": self.Anonfiles, "bayfiles": self.Anonfiles, "bunkr": self.Bunkr,
+        self.mapping = {"anonfiles.com": self.Anonfiles, "bayfiles": self.Anonfiles, "xbunkr": self.XBunkr,
+                        "bunkr": self.Bunkr,
                         "coomer.party": self.coomer, "cyberdrop": self.Cyberdrop, "cyberfile.is": self.cyberfile,
                         "erome.com": self.Erome, "gfycat.com": self.gfycat, "gofile.io": self.GoFile,
                         "img.kiwi": self.ShareX, "jpg.church": self.ShareX, "jpg.homes": self.ShareX,
                         "kemono.party": self.Kemono, "pixeldrain.com": self.Pixeldrain, "pixl.is": self.ShareX,
                         "postimg": self.Postimg, "putme.ga": self.ShareX, "putmega.com": self.ShareX,
-                        "redgifs.com": self.redgifs, "saint.to": self.Saint, "socialmediagirls": self.SocialMediaGirls,
+                        "redgifs.com": self.redgifs, "rule34.xxx": self.Rule34, "saint.to": self.Saint, "socialmediagirls": self.SocialMediaGirls,
                         "simpcity": self.SimpCity, "xbunker": self.XBunker}
 
     async def Anonfiles(self, url: URL, title=None):
@@ -202,6 +215,19 @@ class ScrapeMapper():
                 await self.Cascade.add_to_album("redgifs.com", "gifs", content_url, url)
         await redgifs_session.exit_handler()
 
+    async def Rule34(self, url: URL, title=None):
+        rule34_session = Session(self.client)
+        if not self.rule34_crawler:
+            self.rule34_crawler = Rule34Crawler()
+        domain_obj = await self.rule34_crawler.fetch(rule34_session, url)
+        
+        if title:
+            await domain_obj.append_title(title)
+        await self.Cascade.add_albums(domain_obj)
+            
+        await rule34_session.exit_handler()
+
+
     async def Saint(self, url: URL, title=None):
         saint_session = Session(self.client)
         if not self.saint_crawler:
@@ -259,6 +285,28 @@ class ScrapeMapper():
             await self.Cascade.extend(await self.xbunker_crawler.fetch(xbunker_session, url))
         await xbunker_session.exit_handler()
 
+    async def XBunkr(self, url: URL, title=None):
+        xbunkr_session = Session(self.client)
+        if not self.xbunkr_crawler:
+            self.xbunkr_crawler = XBunkrCrawler(include_id=self.include_id)
+        domain_obj = await self.xbunkr_crawler.fetch(xbunkr_session, url)
+        if title:
+            await domain_obj.append_title(title)
+        await self.Cascade.add_albums(domain_obj)
+        await xbunkr_session.exit_handler()
+
+    async def jdownloader_setup(self):
+        try:
+            if not self.jdownloader_auth.username or not self.jdownloader_auth.password or not self.jdownloader_device:
+                raise Exception("jdownloader credentials were not provided.")
+            jd = myjdapi.Myjdapi()
+            jd.set_app_key("CYBERDROP-DL")
+            jd.connect(self.jdownloader_auth.username, self.jdownloader_auth.password)
+            self.jdownloader_agent = jd.get_device(self.jdownloader_device)
+        except:
+            await log("Failed jdownloader setup")
+            self.jdownloader_enable = False
+
     async def map_url(self, url_to_map: URL, title=None):
         if not url_to_map:
             return
@@ -272,6 +320,18 @@ class ScrapeMapper():
                 else:
                     await log("Skipping scrape of " + str(url_to_map))
                 return
-        await log(str(url_to_map) + " is not supported currently.")
-        async with aiofiles.open("./Unsupported_Urls.txt", mode='a') as f:
-            await f.write(str(url_to_map)+"\n")
+
+        if self.jdownloader_enable:
+            if not self.jdownloader_agent:
+                await self.jdownloader_setup()
+            try:
+                if "facebook" in url_to_map.host.lower() or "instagram" in url_to_map.host.lower():
+                    raise Exception("Blacklisted META")
+                await log("Sending " + str(url_to_map) + " to JDownloader")
+                self.jdownloader_agent.linkgrabber.add_links([{"autostart": False, "links": str(url_to_map)}])
+            except:
+                await log("Failed to send " + str(url_to_map) + " to JDownloader")
+        else:
+            await log(str(url_to_map) + " is not supported currently.")
+            async with aiofiles.open("./Unsupported_Urls.txt", mode='a') as f:
+                await f.write(str(url_to_map)+"\n")
